#ifndef __MY_COMMON__
#define __MY_COMMON__

#include <stdio.h>
#include "chuck_fft.h"

// This files contains some basic definitions to be used
// throughout theh project

// Define some common types
typedef double MyFloat64;
typedef float MyFloat32;
typedef unsigned int MyUint32;
typedef unsigned char MyUint8;

typedef MyFloat32 Sample;

typedef struct point {
  float x;
  float y;
} point;


#define MAX(a,b)    ((a)<(b)?(b):(a))
#define MIN(a,b)    ((a)<(b)?(a):(b))

#define MY_PI 3.14159265358979

// Debug utilities
#ifdef SND_DEBUG_ON
  #define DEBUG_PRINTF(x, ...)        do { printf(x, ## __VA_ARGS__); fflush(stdout); } while(0);
#else  
  #define DEBUG_PRINTF
#endif //POLY_DEBUG_ON

#ifdef DUMP_MIDI
  #define MIDI_DEBUG(x, ...)          do { printf(x, ## __VA_ARGS__); fflush(stdout); } while(0);
#else  
  #define MIDI_DEBUG
#endif //DUMP_MIDI


#ifdef WIN32
#pragma warning(disable : 4996)
#endif


/********************************************************************************* 
 OS-Specific Stuff
**********************************************************************************/

// Operating system dependent thread functionality.
#if defined(__WINDOWS_DS__) || defined(__WINDOWS_ASIO__)
#include <windows.h>
#include <process.h>

// Mutex related definition
typedef CRITICAL_SECTION OsMutex;
#define MUTEX_INITIALIZE(A) InitializeCriticalSection(A)
#define MUTEX_DESTROY(A)    DeleteCriticalSection(A)
#define MUTEX_LOCK(A)       EnterCriticalSection(A)
#define MUTEX_UNLOCK(A)     LeaveCriticalSection(A)

#define OS_SLEEP( X ) Sleep(X)
#define RT_AUDIO_API_TYPE     (RtAudio::WINDOWS_DS)

#endif

#if defined(__LINUX_ALSA__) || defined(__UNIX_JACK__) || defined(__LINUX_OSS__) || defined(__MACOSX_CORE__)
#include <unistd.h>  
#include <pthread.h>
// Mutex related definitions
typedef pthread_mutex_t OsMutex;
#define MUTEX_INITIALIZE(A) pthread_mutex_init(A, NULL)
#define MUTEX_DESTROY(A)    pthread_mutex_destroy(A)
#define MUTEX_LOCK(A)       pthread_mutex_lock(A)
#define MUTEX_UNLOCK(A)     pthread_mutex_unlock(A)

#define OS_SLEEP( X ) usleep(X)

#if defined (__LINUX_ALSA__)
#define RT_AUDIO_API_TYPE     (RtAudio::LINUX_ALSA)
#elif defined (__UNIX_JACK__)
#define RT_AUDIO_API_TYPE     (RtAudio::UNIX_JACK)
#else
#error
#endif

#endif



#endif //__MY_COMMON__

