////////////////////////////////////////////////////////////////////////////
// File Name:   audio_engine.h
// Description: Audio engine
// Author:      Roy Fejgin
// Date:        Autumn 2009
////////////////////////////////////////////////////////////////////////////
#ifndef __AUDIO_ENG__
#define __AUDIO_ENG__

#include <list>
#include "Stk.h"
#include "FileWvIn.h"
#include "RtAudio.h"
#include "chuck_fft.h"
#include "Common.h"

using namespace std;

class Range {
public:
  Range() : start(0), end(1){}
  Range(float s, float e) : start(s), end(e){}
  float start;
  float end;
};

typedef struct RangeInfo {
  Range freq;
  Range time;
} RangeInfo;

class AudioEngine
{
public:
  static const int TYPE_TIME = 0;
  static const int TYPE_FREQ = 1;
  static const int TYPE_PHASE = 2;  
  static const int TYPE_TOTAL = 3;  

  // Constructor
  AudioEngine(int fftSize, OsMutex* pMutex, char* fileName);

  // Destructor
  // TODO
  ~AudioEngine(){}

  void renderAudio(Sample* audioOutBuf, Sample* audioInBuf, Sample* fftOutBuf, int nFrames);
  void setEnvelope(int type, list<point>& pEnvelopeInput,
    float maxX, float minX,
    float maxY, float minY);
  void setRange(float start, float end, int type);  
  void loadWavFile(const char* fileName);

private:
  void initEnvelope(Sample* pBuf, Sample value, int len);
  void calcEnvelope(float* pEnvelope, int length,  list<point>& envelopeInput,
                               float env_maxX, float env_minX,
                               float env_maxY, float env_minY,
                               float rangeStart, float rangeEnd);

  float leakyIntegrate(float oldVal, float newVal);
  void multBuffers(Sample* bufIn, Sample* bufMult, long length);

  // Time domain envelope
  Sample * m_timeEnvelope;
  // Frequency domain envelope
  Sample * m_freqEnvelope;
  // Phase envelope
  Sample * m_phaseEnvelope;
  // Hanning Window. Time domain.
  Sample * m_HanningBuffer;

  int m_bufferSize;

  // Last two windows of time-domain audio data
  Sample * m_WindowedBuffers[2];
  int m_winBufIndex;

  // FFT size
  long m_fftSize;

  // WAV input file tracking
  Sample * m_sampleBuffer;
  int m_currSample;
  int m_numSamples;
  int m_numWindows;

  // Ranges
  RangeInfo m_ranges[TYPE_TOTAL];
  float m_lastRangeStart;
  float m_lastRangeEnd;
  int m_lastRangeType;

  // Mutex
  OsMutex* m_pMutex;
};

#endif //__AUDIO_ENG__

