////////////////////////////////////////////////////////////////////////////
// File Name:   ui_envelope.h
// Description: Button GUI element
// Author:      Roy Fejgin
// Date:        Autumn 2009
////////////////////////////////////////////////////////////////////////////
#ifndef __UI_BUTTON__
#define __UI_BUTTON__

#include "ui_element.h"
#include "ui_util.h"

class UI_Button : public UI_Element
{
public:

  // Constructor
  UI_Button(float x, float y, float width, float height, 
            Color buttonColor, Color textColor,
            char* text, void (*callback)(void)) :
          UI_Element(x, x+width, y, y+height, true),
          m_buttonColor(buttonColor), m_textColor(textColor),
          m_text(text), m_callback(callback)
  {
  }

  // Destructor
  ~UI_Button(){};

  virtual void draw() {
    switchToOrtho();

    glColor3f(m_buttonColor.m_r, m_buttonColor.m_g, m_buttonColor.m_b);
    glBegin(GL_QUADS);						        
    glVertex3f(m_xMin, m_yMax, -0.01f);				// Top Left
    glVertex3f(m_xMax, m_yMax, -0.01f);				// Top Right
    glVertex3f(m_xMax, m_yMin, -0.01);				// Bottom Right
    glVertex3f(m_xMin, m_yMin, -0.01f);				// Bottom Left
    glEnd();

    glColor3f(m_textColor.m_r, m_textColor.m_g, m_textColor.m_b);
    displayString(m_xMin+0.05, m_yMin+(m_yMax-m_yMin)/2 - 0.03, m_text);

    switchToPerspective();
  }

  // Call callback on mouse click
  virtual void mouseDown(point p ) {m_callback();}

protected:
  void (*m_callback)(void);
  char* m_text;
  Color m_buttonColor;
  Color m_textColor;
};

#endif //__UI_BUTTON__

