////////////////////////////////////////////////////////////////////////////
// File Name:   ui_envelope.cpp
// Description: Envelope GUI element
// Author:      Roy Fejgin
// Date:        Autumn 2009
////////////////////////////////////////////////////////////////////////////

#include "ui_envelope.h"
#include <list>

void UI_Envelope::draw()
{
  // Switch to Ortho projection
  switchToOrtho();
  
  // Draw
  drawEnv();
  drawAxis();
  
  // Go back to perspective projection
  switchToPerspective();
}


void UI_Envelope::drawEnv()
{
  glLineWidth(4.0);

  // Draw the envelope
  glBegin( GL_LINE_STRIP );
  glColor3f(PINK(1.0));
  for( list<point>::iterator  iter = m_points.begin(); iter!= m_points.end(); ++iter) {
    point p = *iter;
    // Draw vertex
    if (p.x  >= m_xMin && p.x <= m_xMax) {
      p.y = MAX(m_yMin, p.y);
      glVertex2f(p.x, p.y); 
    }
  }

  // Finish line
  glEnd();
}

// Draw X and Y axis
void UI_Envelope::drawAxis()
{
  glLineWidth(3.2);

  // Draw the envelope X axis
  glBegin( GL_LINE_STRIP );
  glColor3f(GREEN(1.0));
  glVertex2f(m_xMin,m_yMin);
  glVertex2f(m_xMax,m_yMin);
  glEnd();

  // Draw arrow for X axis
  glBegin( GL_LINE_STRIP );
  glVertex2f(m_xMax-0.05,m_yMin-0.05);
  glVertex2f(m_xMax,m_yMin);
  glEnd();
  glBegin( GL_LINE_STRIP );
  glVertex2f(m_xMax-0.05,m_yMin+0.05);
  glVertex2f(m_xMax,m_yMin);
  glEnd();

  // Draw the envelope Y axis
  glBegin( GL_LINE_STRIP );
  glColor3f(GREEN(1.0));
  glVertex2f(m_xMin,m_yMin);
  glVertex2f(m_xMin,m_yMax);
  glEnd();
 
  // Draw arrow for Y axis
  glBegin( GL_LINE_STRIP );
  glVertex2f(m_xMin-0.03,m_yMax-0.05);
  glVertex2f(m_xMin,m_yMax);
  glEnd();
  glBegin( GL_LINE_STRIP );
  glVertex2f(m_xMin+0.03,m_yMax-0.05);
  glVertex2f(m_xMin,m_yMax);
  glEnd();

  // Draw axis labels
  glColor3f(m_textColor.m_r, m_textColor.m_g, m_textColor.m_b);
  displayString(m_xMin+(m_xMax-m_xMin)/2, m_yMin-0.1, m_xLabel);
  displayString(m_xMin-0.3, m_yMin+(m_yMax-m_yMin)/2, m_yLabel);


}

void UI_Envelope::randomize()
{
  point p;
  p.x = 0;
  p.y = 0;
  mouseDown(p);
  for(float i = m_xMin; i < m_xMax; i+=0.025) {
    p.x = i;
    p.y = randFloat(m_yMin, m_yMax);
    m_points.push_back(p);
  }
  mouseUp(p);
}

void UI_Envelope::mouseDown(point p)
{
  m_points.clear();
  m_lastMotionX = INVALID_X_COORD;
}

void UI_Envelope::mouseUp(point p)
{
  // Feed new envelope to audio engine
  m_audioEng.setEnvelope(m_envType, m_points,
                         m_xMax, m_xMin,
                         m_yMax, m_yMin);
}

void UI_Envelope::mouseMotion(point p)
{
  if (p.x > m_lastMotionX) {
    m_points.push_back(p);
    m_lastMotionX = p.x;
  }
}
