////////////////////////////////////////////////////////////////////////////
// File Name:   ui_envelope.h
// Description: Envelope GUI element
// Author:      Roy Fejgin
// Date:        Autumn 2009
////////////////////////////////////////////////////////////////////////////
#ifndef __UI_ENVELOPE__
#define __UI_ENVELOPE__

#include "audio_engine.h"
#include "ui_element.h"
#include "ui_util.h"
#include <list>

class UI_Envelope : public UI_Element
{
public:

  // Constructor
  UI_Envelope(AudioEngine& audioRef, float xMin, float xMax, float yMin, 
              float yMax, bool isActive, Color textColor, int envType,
              char* xLabel, char* yLabel)
              : UI_Element(xMin, xMax, yMin, yMax, isActive),
                m_audioEng(audioRef)            
  {
    m_textColor = textColor;
    m_lastMotionX = INVALID_X_COORD;
    m_xLabel = xLabel;
    m_yLabel = yLabel;
    m_envType = envType;
  }

  // Destructor
  ~UI_Envelope(){};

  virtual void draw();
  virtual void mouseDown(point p );
  virtual void mouseUp(point p);
  virtual void mouseMotion(point p);
  
  void setXlabel(char* pChar) { m_xLabel = pChar;};
  void setYlabel(char* pChar) { m_yLabel = pChar;};
  char* getXlabel() {return m_xLabel; }
  void randomize();

protected:
  void drawAxis();
  void drawEnv();

  char* m_xLabel;
  char* m_yLabel;
  Color m_textColor;
  // list of coordinates of the envelope
  list<point> m_points;
  float m_lastMotionX;
  int m_envType;
  AudioEngine& m_audioEng;
};

#endif //__UI_ENVELOPE__

