////////////////////////////////////////////////////////////////////////////
// File Name:   ui_range.cpp
// Description: Range GUI element
// Author:      Roy Fejgin
// Date:        Autumn 2009
////////////////////////////////////////////////////////////////////////////

#include "ui_range.h"
#include "ui_util.h"
#include "string.h"

void UI_Range::draw()
{
  // Switch to Ortho projection
  switchToOrtho();
  
  // Draw
  drawRange();
  drawAxis();
  
  // Go back to perspective projection
  switchToPerspective();
}


void UI_Range::drawRange()
{
  if(m_xStart != INVALID_X_COORD) {
    glLineWidth(4.0);

    glColor3f(YELLOW(1.0));
    glBegin(GL_QUADS);						        
    glVertex3f(m_xStart, m_yMax, -0.001f);				// Top Left
    glVertex3f(m_xEnd, m_yMax, -0.001f);				  // Top Right
    glVertex3f(m_xEnd, m_yMin, -0.001);			  	  // Bottom Right
    glVertex3f(m_xStart, m_yMin, -0.001f);  			// Bottom Left
    glEnd();
  }
}

// Draw X and Y axis
void UI_Range::drawAxis()
{
  glLineWidth(3.2);

  // Draw the envelope X axis
  glBegin( GL_LINE_STRIP );
  glColor3f(BLUE(1.0));
  glVertex2f(m_xMin,m_yMin);
  glVertex2f(m_xMax,m_yMin);
  glEnd();

  // Draw arrow for X axis
  glBegin( GL_LINE_STRIP );
  glVertex2f(m_xMax-0.05,m_yMin-0.05);
  glVertex2f(m_xMax,m_yMin);
  glEnd();
  glBegin( GL_LINE_STRIP );
  glVertex2f(m_xMax-0.05,m_yMin+0.05);
  glVertex2f(m_xMax,m_yMin);
  glEnd();

  // Draw the envelope Y axis
  glBegin( GL_LINE_STRIP );
  glColor3f(BLUE(1.0));
  glVertex2f(m_xMin,m_yMin);
  glVertex2f(m_xMin,m_yMax);
  glEnd();
 
  // Draw arrow for Y axis
  glBegin( GL_LINE_STRIP );
  glVertex2f(m_xMin-0.03,m_yMax-0.05);
  glVertex2f(m_xMin,m_yMax);
  glEnd();
  glBegin( GL_LINE_STRIP );
  glVertex2f(m_xMin+0.03,m_yMax-0.05);
  glVertex2f(m_xMin,m_yMax);
  glEnd();

  // Create label
  char label[100];
  memcpy(label, m_xLabel, strlen(m_xLabel)+1);
  strcat(label, " Range");

  // Draw axis labels
  glColor3f(m_textColor.m_r, m_textColor.m_g, m_textColor.m_b);
  displayString(m_xMin+(m_xMax-m_xMin)/2-0.25, m_yMin-0.1, label);
  //displayString(m_xMin-0.3, m_yMin+(m_yMax-m_yMin)/2, m_yLabel);


}

void UI_Range::mouseDown(point p)
{
  m_xStart = p.x;
  m_xEnd = p.x;
}

void UI_Range::mouseUp(point p)
{
  float start, end;

  if(m_xStart == m_xEnd) {
    // Clicking without dragging clears the range
    start = 0;
    end = 1;
  } else { 
    // Normalize
    start = (m_xStart-m_xMin) / (m_xMax-m_xMin);
    end = (m_xEnd-m_xMin) / (m_xMax-m_xMin);
  }
  m_audioEng.setRange(start, end, m_type);
}

void UI_Range::mouseMotion(point p)
{
  if (p.x > m_xStart && p.x < m_xMax) {
    m_xEnd = p.x;
  }
}
