////////////////////////////////////////////////////////////////////////////
// File Name:   ui_range.h
// Description: Range GUI element
// Author:      Roy Fejgin
// Date:        Autumn 2009
////////////////////////////////////////////////////////////////////////////
#ifndef __UI_RANGE__
#define __UI_RANGE__

#include "audio_engine.h"
#include "ui_element.h"
#include "ui_util.h"

class UI_Range : public UI_Element
{
public:

  // Constructor
  UI_Range(AudioEngine& audioRef, float xMin, float xMax, float yMin, 
              float yMax, bool isActive, Color textColor, int type,
              char* xLabel, char* yLabel)
              : UI_Element(xMin, xMax, yMin, yMax, isActive),
                m_audioEng(audioRef)            
  {
    m_textColor = textColor;
    m_lastMotionX = INVALID_X_COORD;
    m_xStart = m_xEnd = INVALID_X_COORD;
    m_xLabel = xLabel;
    m_yLabel = yLabel;
    m_type = type;
  }

  // Destructor
  ~UI_Range(){};

  virtual void draw();
  virtual void mouseDown(point p );
  virtual void mouseUp(point p);
  virtual void mouseMotion(point p);
  
  void setXlabel(char* pChar) { m_xLabel = pChar;}
  void setYlabel(char* pChar) { m_yLabel = pChar;}
  void setType(int type) {m_type = type;}

  void clear() {
    m_xStart = m_xEnd = m_lastMotionX = INVALID_X_COORD;    
  }

protected:
  void drawAxis();
  void drawRange();

  char* m_xLabel;
  char* m_yLabel;
  Color m_textColor;
  // Selected range
  float m_xStart, m_xEnd;
  float m_lastMotionX;
  int m_type;
  AudioEngine& m_audioEng;
};

#endif //__UI_RANGE__

