////////////////////////////////////////////////////////////////////////////
// File Name:   ui_waterfall.cpp
// Description: Spectrum Waterfall display GUI element
// Author:      Roy Fejgin
// Date:        Autumn 2009
////////////////////////////////////////////////////////////////////////////
#include "ui_waterfall.h"
#include <math.h>

void UI_Waterfall::draw()
{
    // Line width
  glLineWidth( 2.0 );

  // Push the matrix
  glPushMatrix();

  // Vertex coordinates
  GLfloat y = m_yMin;
  GLfloat x = m_xMin;
  GLfloat z = 1.0, zinc=-.1;

  // Calculate x increment
  GLfloat xinc;
  GLfloat logMax = log10(10.0+(GLfloat)m_spectLen/2);
  if (!m_logFreq) {
    xinc = ::fabs(2*x / (m_spectLen/2));
  }    

  // Select spectrum from history
  Sample* pCurrentBuf = m_spectBuffer[m_index];
  int index = m_index;

  GLfloat colorDecay = 0.8;

  glColor3f(ORANGE(1.4));

  // Iterate over all spectrums in the history
  for( int i=0; i<m_historyLen; i++) {
    pCurrentBuf = m_spectBuffer[index];
    GLfloat xFrac = 0.0, nextXFrac = 0.0;

    x = m_xMin;

    // Logarithmic x (frequency) axis  
    if (m_logFreq) {
      xFrac = getXFraction(0, logMax);
      nextXFrac = getXFraction(1, logMax);
      xinc = 2*(nextXFrac - xFrac);
    }

    // Start the line
    glBegin( GL_LINE_STRIP );        


    // loop through the spectrum
    for( int j = 0; j < m_spectLen/2; j++ )
    {
      y = pCurrentBuf[j];
      y+= 10;
      y = log10(y);
      y = m_scalingFactor*(y-1.0);

      // Set the next vertex
      glVertex3f( x, y+1.8, z);
      //glVertex2f( x, y+i*0.03);

      // Logarithmic x (frequency) axis
      if (m_logFreq) {        
        xinc = ::fabs(3.159*m_xMin*(nextXFrac - xFrac));
        xFrac = nextXFrac;
        nextXFrac = getXFraction(j+1, logMax);
      }

      // Increment x
      x += xinc;//*pow(0.99,i);
    }

    // Finish line
    glEnd();

    z += zinc;

    index--;
    if (index< 0) {
      index = m_historyLen - 1;
    }

    // Fade out the color for older spectrums
    colorDecay *= COLOR_DECAY_RATE;

    // set the color for the next spectrum
    glColor3f(ORANGE(colorDecay));
  }

  // Next time we'll overwrite the oldest spectrum in the history
  m_index = (m_index + 1) % m_historyLen;

  // pop
  glPopMatrix();

}
void UI_Waterfall::insertSpectrum(Sample* pSpectrum)
{
  Sample* pBuf = m_spectBuffer[m_index];

  // Copy FFT results
  memcpy(pBuf, pSpectrum, m_spectLen*sizeof(Sample));

  // Calculate magnitudes
  for (int i=0; i< m_spectLen; i+=2) {
    // main FFT buffer magnitude
    complex freqBin = *((complex*)(void*)&pBuf[i]);
    pBuf[i/2] = cmp_abs(freqBin);
  }
}
