////////////////////////////////////////////////////////////////////////////
// File Name:   ui_waterfall.h
// Description: Spectrum Waterfall display GUI element
// Author:      Roy Fejgin
// Date:        Autumn 2009
////////////////////////////////////////////////////////////////////////////
#ifndef __UI_WATERFALL__
#define __UI_WATERFALL__

#include <list>
#include <math.h>

#include "ui_element.h"
#include "ui_util.h"
#include "Common.h"


#define SCALING_FACTOR  (1300.0f)


class UI_Waterfall : public UI_Element
{
public:

  // Constructor
  UI_Waterfall(float xStart, float yStart, bool isActive, int historyLen, 
              int spectLen)
              : UI_Element(xStart, xStart, yStart, yStart, isActive)
  {
    m_historyLen = historyLen;
    m_spectLen = spectLen;
    m_logFreq = false;
    m_index = 0;
    m_scalingFactor = SCALING_FACTOR;
    m_spectBuffer = new Sample*[historyLen];
    for (int i=0; i<m_historyLen; i++) {
      m_spectBuffer[i] = new Sample[m_spectLen];
      memset(m_spectBuffer[i] , 0, m_spectLen*sizeof(Sample));
    }   
  }

  // Destructor
  ~UI_Waterfall() { 
    for(int i=0; i<m_historyLen; i++) {
      delete m_spectBuffer[i];
    }
    delete[] m_spectBuffer;    
  }; 

  virtual void draw();
  void insertSpectrum(Sample* pSpectrum);


protected:

  float getXFraction(int n, float logMax) 
  {
    return log10((float)n+10.0)/logMax;
  }

  Sample** m_spectBuffer;
  int m_index;
  int m_historyLen;
  int m_spectLen;  
  bool m_logFreq;
  float m_scalingFactor;
};

#endif //__UI_WATERFALL__

